const { app, BrowserWindow, dialog, Menu } = require('electron');
const fs = require("fs");
const {ipcMain} = require("electron");
const Store = require("electron-store");

var jsdom = require("jsdom");
const { JSDOM } = jsdom;
const { window } = new JSDOM();
const { document } = (new JSDOM('')).window;
const store = new Store();
global.document = document;

var $ = jQuery = require('jquery')(window);

let mainWindow = null;
let splash_screen = null;
let save_data = {
  file_path: null
}
let menu_template = [
  {
    label: "File",
    submenu: [
      {
        label: "Save",
        accelerator: "Ctrl+S",
        click: function () {
          mainWindow.webContents.send("save", false)
        }
      },
      {
        label: "Save As",
        accelerator: "Ctrl+Shift+S",
        click: function () {
          mainWindow.webContents.send("save", true)
        }
      },
      {
        label: "Open",
        accelerator: "Ctrl+O",
        click: openFile
      },
      {
        label: "Open Recent",
        submenu:[]
      },
      {
        label: "Exit",
        click: function () {
          app.quit();
        }
      }
    ]
  }
]

function openFile() {
  let options = {
    //Placeholder 1
    title: "Open website data",

    //Placeholder 2
    defaultPath : store.get("default_dir", app.getPath("documents")),

    //Placeholder 3
    filters :[
      {name: 'json', extensions: ['json']},
      {name: 'All Files', extensions: ['*']}
    ]
  }
  let file = dialog.showOpenDialogSync(options);
  if (file) {
    store.set("default_dir", file[0])
    save_data.file_path = file[0];
    if (mainWindow) {
      mainWindow.webContents.send("open", save_data.file_path);
    } else {
      loadFile(file[0]);
    }
  }
}

function loadFile(file_name) {
  save_data.file_path = file_name;
  createWindow();
  mainWindow.webContents.once("dom-ready", () => {
    mainWindow.webContents.send("open", file_name);
    splash_screen.close();
  })
}

function sanitizeName(name) {
  let new_name = name.toLowerCase();
  for (let char of name) {
    if (char === " ") {
      new_name = new_name.replace(char, "");
    }
  }
  return new_name;
}

function cleanData(data) {
  for (let table of data) {
    let fields = [];
    for (let col of table["col-def"]) {
      fields.push(col["field"]);
    }
    for (let row of table["rows"]) {
      for (let key of Object.keys(row)) {

        if (!fields.includes(key) && key != "uid") {
          delete row[key];
        }
      }
    }
  }
  return data;
}

ipcMain.on("save", (event, data, new_file) => {
  let default_path = app.getPath("documents");
  if (save_data.file_path) {
    default_path = save_data.file_path;
  }
  let options = {
    //Placeholder 1
    title: "Save website data",

    //Placeholder 2
    defaultPath : default_path,

    //Placeholder 3
    filters :[
      {name: 'json', extensions: ['json']},
      {name: 'All Files', extensions: ['*']}
    ]
  }

  if (new_file || !save_data.file_path) {
    let path = dialog.showSaveDialogSync(options);
    if (path) {
      save_data.file_path = path;
    }
  }
  if (save_data.file_path) {
    fs.writeFileSync(app.getPath("userData")+"\\data.json", JSON.stringify(save_data))
    data = cleanData(data);
    fs.writeFileSync(save_data.file_path, JSON.stringify(data, null, 4));
    let name = save_data.file_path.match(/.+\\([^\\]+)/)[1];
    let nav_content = save_data.file_path.replace(".json", "-nav-content.html");
    let tab_content = save_data.file_path.replace(".json", "-tab-content.html");
    let tabs = "";
    let nav = "";

    let table_tabs = {"none": []}

    for (let table of data) {
      if (table["tab-name"] == "") {
        table_tabs["none"].push(table);
      } else {
        if (!Object.keys(table_tabs).includes(table["tab-name"])) {
          table_tabs[table["tab-name"]] = [table];
        } else {
          table_tabs[table["tab-name"]].push(table);
        }
      }
    }

    let added_tables = []
    for (let table of data) {
      if (!table["hidden"]) {
        if (!added_tables.includes(table["name"])) {
          if (table["tab-name"] == "") {
            tabs += $("<div></div>")
                .addClass("tab-pane fade")
                .attr("id", sanitizeName(table["name"]))
                .attr("role", "tabpanel")
                .attr("aria-labelledby", sanitizeName(table["name"])+"-tab")
                .append($("<h3></h3>")
                    .attr("id", sanitizeName(table["name"])+"-header")
                    .append(`${table["name"]} (`)
                    .append($("<span></span>")
                        .attr("id", sanitizeName(table["name"])+"-completed"))
                    .append("/")
                    .append($("<span></span>")
                        .attr("id", sanitizeName(table["name"])+"-total"))
                    .append(")"))
                .append($("<p></p>")
                    .html(table["description"]))
                .append($("<table></table>")
                    .attr("id", sanitizeName(table["name"])+"-table")
                    .addClass("table table-hover"))
                .html();
            nav += $("<li></li>")
                .addClass("nav-item")
                .append($("<a></a>")
                    .addClass("nav-link")
                    .attr("id", sanitizeName(table["name"])+"-tab")
                    .attr("data-toggle", "tab")
                    .attr("href", "#"+sanitizeName(table["name"]))
                    .attr("role", "tab")
                    .attr("aria-controls", sanitizeName(table["name"]))
                    .attr("aria-selected", false)
                    .html(table["name"]))
                .html();

            added_tables.push(table["name"]);
          } else {
            let tab_name = table["tab-name"];
            let a = $("<a></a>")
                .attr("id", sanitizeName(tab_name)+"-tab")
                .addClass("nav-link dropdown-toggle")
                .attr("data-toggle", "dropdown")
                .attr("href", "#")
                .attr("role", "button")
                .attr("aria-haspopup", true)
                .attr("aria-expanded", false)
                .html(tab_name);
            let div = $("<div></div>")
                .addClass("dropdown-menu")
                .attr("x-placement", "bottom-start")
                .css({"position": "absolute", "will-change": "transform", "top": "0", "left": "0", "transform": "translate3d(0px, 39px, 0px)"});
            for (let tab_table of table_tabs[tab_name]) {
              tabs += $("<div></div>")
                  .addClass("tab-pane fade")
                  .attr("id", sanitizeName(tab_table["name"]))
                  .attr("role", "tabpanel")
                  .attr("aria-labelledby", sanitizeName(table["name"])+"-tab")
                  .append($("<h3></h3>")
                      .attr("id", sanitizeName(tab_table["name"])+"-header")
                      .append(`${tab_table["name"]} (`)
                      .append($("<span></span>")
                          .attr("id", sanitizeName(tab_table["name"])+"-completed"))
                      .append("/")
                      .append($("<span></span>")
                          .attr("id", sanitizeName(tab_table["name"])+"-total"))
                      .append(")"))
                  .append($("<p></p>")
                      .html(tab_table["description"]))
                  .append($("<table></table>")
                      .attr("id", sanitizeName(tab_table["name"])+"-table")
                      .addClass("table table-hover"))
                  .html();
              div.append("<a></a>")
                  .addClass("dropdown-item")
                  .attr("data-toggle", "tab")
                  .attr("href", "#"+sanitizeName(tab_table["name"]))
                  .attr("aria-expanded", true)
                  .html(tab_table["name"]);
              added_tables.push(tab_table["name"]);
            }
            let li = $("<li></li>")
                .addClass("nav-item dropdown");
            li.append(a);
            li.append(div);
            nav += li.prop('outerHTML');
          }
        }
      }
    }
    fs.writeFileSync(nav_content, nav);
    fs.writeFileSync(tab_content, tabs);
  }
})

ipcMain.on("loadFile", (event, file_name) => {
  loadFile(file_name);
})

ipcMain.on("newFile", (event) => {
  createWindow();
  mainWindow.webContents.once("dom-ready", () => {
    splash_screen.close();
  })
})

ipcMain.on("openFile", (event) => {
  openFile();
})

ipcMain.handle("getStoreValue", (event, key) => {
  return store.get(key);
})

ipcMain.handle("setStoreValue", (event, key, value) => {
  store.set(key, value);
})

// Handle creating/removing shortcuts on Windows when installing/uninstalling.
if (require('electron-squirrel-startup')) { // eslint-disable-line global-require
  app.quit();
}

const createWindow = () => {
  // Create the browser window.
  mainWindow = new BrowserWindow({
    width: 900,
    height: 700,
    webPreferences: {
      nodeIntegration: true
    }
  });

  let recent_files = store.get("recent_files");
  for (let menu of menu_template[0].submenu) {
    if (menu.label == "Open Recent") {
      for (let file of recent_files) {
        if (fs.existsSync(file)) {
          menu.submenu.push({
            label: file, click: () => {
              mainWindow.webContents.send("open", file);
            }
          })
        }
      }
    }
  }

  let menu = Menu.buildFromTemplate(menu_template);
  Menu.setApplicationMenu(menu);

  // and load the index.html of the app.
  mainWindow.loadFile('src/index.html');

  // Open the DevTools.
  // mainWindow.webContents.openDevTools();
};

const splashScreen = () => {
  // Create the browser window.
  splash_screen = new BrowserWindow({
    width: 806,
    height: 529,
    webPreferences: {
      nodeIntegration: true
    },
    autoHideMenuBar: true,
    resizable: false,
    frame: true
  });

  let menu = Menu.buildFromTemplate(menu_template);
  Menu.setApplicationMenu(menu);

  if (!store.has("recent_files")) {
    store.set("recent_files", []);
  }

  // and load the index.html of the app.
  splash_screen.loadFile('src/splash.html');

  // Open the DevTools.
  // splash_screen.webContents.openDevTools();
  // store.openInEditor();
};

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', splashScreen);

// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('activate', () => {
  // On OS X it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (BrowserWindow.getAllWindows().length === 0) {
    createWindow();
  }
});

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and import them here.
