const {ipcRenderer} = require("electron")
let data;
let selected = null;
let current_table = null;
let current_col = null;
let current_row = null;
let uid_regex = /(.+)-row(\d+)/;
$.ajax({
	dataType: "json",
	url: "database.json",
	cache: false,
	success: function(json, textStatus, jqXHR ) {
		data = json;
		refreshTables();
		showTable(sanitizeName(data[0]["name"]));
	}
});

function sanitizeName(table_name) {
	let new_name = table_name.toLowerCase();
	for (let char of table_name) {
		if (char === " ") {
			new_name = new_name.replace(char, "");
		}
	}
	return new_name;
}

function replaceLast(str, str1, str2) {
	let last_index = str.lastIndexOf(str1);
	str = str.slice(0, last_index) + str.slice(last_index).replace(str1, str2);
	return str;
}

function makeTables() {
	for (let table of data) {
		let name = sanitizeName(table["name"]);
		let table_index = data.indexOf(table);

		// create nav element
		let btn = $("<button></button>")
			.attr("id", name+"-nav")
			.addClass("w3-bar-item w3-button tablink")
			.css("white-space", "nowrap")
			.on("click", () => {
				showTable(name);
			})
			.on("contextmenu", (event) => {
				navContext(event, table["name"]);
			})
			.attr("draggable", true)
			.on("drop", (event) => {
				tableDrop(event, table_index);
			})
			.on("dragstart", (event) => {
				tableDrag(event, table_index);
			})
			.on("dragover", (event) => {
				allowDrop(event)
			})
			.text(table["name"]);

		$("nav").append(btn);

		let textarea = $(`<textarea></textarea>`)
			.focusout(function (event){save()})
			.attr("id", name+"-description")
			.addClass("table-description")
			.text(table["description"]);

		let table_elm = $("<table></table>");
		table_elm.attr("id", name+"-table")
			.addClass("editor-table");

		let cols = table["col-def"]

		let tr = $("<tr></tr>");
		let td = $("<td></td>");
		td.css("border", "none");
		if (data.length > 1) {
			btn = $("<button>Delete Table</button>")
				.addClass("w3-button w3-red")
				.on("click", () => {
					deleteTable(table["name"]);
				});
			td.html(btn);
		}
		tr.append(td);

		let index = 0;
		for (let col of cols) {
			let id = name+"-"+index.toString();
			let th = $("<th></th>")
				.attr("id", id)
				.addClass("w3-display-container")
				.on("click", (event) => {
					selectCell(table["name"], col["title"], col["field"], id);
				})
				.attr("draggable", true).
				on("drop", {index:index}, (event) => {
					colDrop(event, name, event.data.index);
				})
				.on("dragstart", {index:index}, (event) => {
					drag(event, event.data.index);
				})
				.on("dragover", (event) => {
					allowDrop(event);
				})
				.on("dblclick", () => {
					startEditHeader(id);
				})
				.append($("<span></span>")
					.css({"margin-right": "44px", "white-space": "nowrap"})
					.text(col.title))
				.append($(`<span>&times;</span>`)
					.addClass("w3-button w3-display-right w3-hover-red")
					.on("click", {index:index}, (event) => {
						console.log(table["name"], event.data.index)
						deleteCol(table["name"], event.data.index);
					}));

			tr.append(th)
			index++;
		}

		td = $("<td></td>")
			.css("border", "none")
			.html($("<button>Add Column</button>")
				.addClass("w3-button w3-green")
				.on("click", () => {
					addColumn(table["name"]);
				}));
		tr.append(td);
		table_elm.append(tr);
		for (let row of table["rows"]) {
			let index = table["rows"].indexOf(row);
			tr = $("<tr></tr>")
				.attr("id", row["uid"])
				.on("drop", (event) => {
					rowDrop(event, name, index);
				})
				.on("dragover", (event) => {
					allowDrop(event);
				})

			td = $("<td></td>")
				.css("border", "none")
				.append($("<button>×</button>")
					.addClass("w3-button w3-tiny w3-red")
					.on("click", () => {
						deleteRow(table["name"], index);
					}))
				.append($("<div>☰</div>")
					.css({"display": "inline", "float": "right", "font-size": "2em"})
					.attr("draggable", true)
					.on("dragstart", (event) => {
						rowDrag(event, index);
					}));
			tr.append(td);

			for (let col of cols) {
				let uid = row["uid"]+"-"+col["field"];
				td = $("<td></td>")
					.html(row[col["field"]])
					.attr("id", uid)
					.on("click", () => {
						selectCell(table["name"], col["title"], col["field"], uid);
					})
					.on("dblclick", () => {
						startEditCell(uid);
					});

				tr.append(td);
			}
			table_elm.append(tr);
		}

		tr = $("<tr></tr>");
		td = $("<td></td>")
			.css("border", "none")
			.html($(`<button>Add Row</button>`)
				.addClass("w3-button w3-green")
				.on("click", () => {
					addRow(name);
				}));
		tr.append(td);
		table_elm.append(tr);

		// table_elm.attr("class", "table table-hover");
		$("#tables").append(textarea)
			.append(table_elm);
	}
	$("nav").append($("<button>+</button>")
		.addClass("w3-button w3-green")
		.on("click", () => {
			addTable();
		}));
}

function removeTables() {
	$("#nav").empty();
	$("#tables").empty();
}

function refreshTables() {
	removeTables();
	makeTables();
	if (current_table != null) {
		showTable(current_table);
	}
}

function addColumn(table_name) {
	for (let table of data) {
		if (table["name"] == table_name) {
			let def_name = "Column";
			let names = [];
			for (let i of table["col-def"]) {
				names.push(i["title"]);
			}

			let num = 1;
			let name = `${def_name} ${num}`;
			for (let i in names) {
				if (names.includes(name)) {
					num++;
					name = `${def_name} ${num}`;
				}
			}

			let field = name.toLowerCase().replace(" ", "");
			table["col-def"].push({
				"field": field,
				"title": name
			})

			for (let row of table["rows"]) {
				row[field] = "";
			}
		}
	}
	refreshTables();
}

function addRow(table_name) {
	for (let table of data) {
		if (sanitizeName(table["name"]) == table_name) {
			let row = {};
			for (let col of table["col-def"]) {
				row[col["field"]] = "";
			}

			let uids = [];
			for (let i of table["rows"]) {
				uids.push(i["uid"]);
			}

			let index = 0;
			let uid = `${table_name}-row${index}`
			for (let i in uids) {
				if (uids.includes(uid)) {
					index++;
					uid = `${table_name}-row${index}`;
				}
			}
			row["uid"] = uid;
			table["rows"].push(row);
		}
	}
	refreshTables();
}

function addTable(dup_table=null, tab_name=null) {
	let def_name = "Table";
	let names = [];
	for (let table of data) {
		names.push(table["name"]);
	}

	let num = 1;
	let name = `${def_name} ${num}`;
	for (let i in names) {
		if (names.includes(name)) {
			num++;
			name = `${def_name} ${num}`;
		}
	}

	let col_def = [
		{
			"field": "column1",
			"title": "Column 1",
			"sortable": false,
			"selectable": false
		},
		{
			"field": "column2",
			"title": "Column 2",
			"sortable": false,
			"selectable": false
		}
	];

	let rows = [
		{
			"uid": sanitizeName(name)+"-row0",
			"column1": "",
			"column2": "",
		}
	];

	let new_table = {
		name: name,
		filter: false,
		hidden: false,
		description: "",
		"col-def": col_def,
		rows: rows
	}
	if (dup_table) {
		new_table = JSON.parse(JSON.stringify(dup_table));
		def_name = sanitizeName(new_table.name);
		new_table.name = "copy of " + new_table.name;

		let num = 1;
		let name = new_table.name;
		for (let i in names) {
			if (names.includes(new_table.name)) {
				num++;
				new_table.name = `${name} - ${num}`;
			}
		}

		for (let i = 0; i < new_table["rows"].length; i++) {
			new_table["rows"][i]["uid"] = new_table["rows"][i]["uid"].replace(def_name, sanitizeName(new_table.name));
		}
	}

	data.push(new_table)
	refreshTables();
}

function selectCell(table_name, title, field, cell_id) {
	let elm = $(document.getElementById(cell_id));
	if (elm.is("th")) {
		$("#cell-text").prop("disabled", true);
		$("#cell-text").val("");
	}
	else {
		$("#cell-text").prop("disabled", false);
	}
	selected = {table_name:table_name,title:title,field:field, cell:cell_id};
	current_col = {"title":title, "field":field};
	$("td").removeClass("w3-green");
	$("th").removeClass("w3-green");
	elm.addClass("w3-green");
	for (let table of data) {
		if (table["name"] == table_name) {
			$("#tab-name").val(table["tab-name"]);
			$("#table-name").val(table_name);
			$("#table-hidden").prop("checked", table["hidden"]);
			$("#col-name").val(title);
			for (let col of table["col-def"]) {
				if (col["title"] == title) {
					$("#col-sortable").prop("checked", col["sortable"]);
					$("#col-selectable").prop("checked", col["selectable"]);
				}
			}
			for (let row of table["rows"]) {
				if (row["uid"]+`-${field}` == cell_id) {
					$("#cell-text").val(row[field]);
				}
			}
		}
	}
}

function save() {
	if (selected != null) {
		let tab_name = $("#tab-name").val();
		let table_name = $("#table-name").val();
		let table_hidden = $("#table-hidden").prop("checked");
		let title = $("#col-name").val();
		let field = sanitizeName($("#col-name").val());
		let sortable = $("#col-sortable").prop("checked");
		let selectable = $("#col-selectable").prop("checked");
		let cell_text = $("#cell-text").val();
		for (let table of data) {
			if (table["name"] == selected["table_name"]) {
				if (table["name"] != table_name) {
					let table_exsits = false;
					for (let other_table of data) {
						if (other_table["name"] == table_name) {
							table_exsits = true;
						}
					}
					if (!table_exsits && table_name.length > 0) {
						selected["cell"] = selected["cell"].replace(sanitizeName(table["name"]), sanitizeName(table_name));
						table["name"] = table_name;
						for (let row of table["rows"]) {
							let match = row["uid"].match(uid_regex);
							row["uid"] = `${sanitizeName(table_name)}-row${match[2]}`;
						}
						$("#table-name").removeClass("w3-red");
					} else {
						$("#table-name").addClass("w3-red");
						return;
					}
				}
				let table_description = $(document.getElementById(`${sanitizeName(table["name"])}-description`));
				table["description"] = table_description.val();
				table["tab-name"] = tab_name;
				table["hidden"] = table_hidden;
				for (let col of table["col-def"]) {
					if (col["field"] == selected["field"]) {
						if (col["field"] != field) {
							let col_exists = false;
							for (let other_col of table["col-def"]) {
								if (other_col["field"] == field) {
									col_exists = true;
								}
							}

							if (!col_exists && field.length > 0) {
								for (let row of table["rows"]) {
									console.log(field);
									console.log(col["field"]);
									row[field] = row[col["field"]];
									delete row[col["field"]];
								}
								selected["cell"] = replaceLast(selected["cell"], col["field"], field);
								col["title"] = title;
								col["field"] = field;
								$("#col-name").removeClass("w3-red");
							} else {
								$("#col-name").addClass("w3-red");
								return;
							}
						}
						col["sortable"] = sortable;
						col["selectable"] = selectable;
					}
				}
				for (let row of table["rows"]) {
					if (row["uid"]+"-"+field == selected["cell"]) {
						row[selected["field"]] = cell_text;
					}
				}
			}
		}
		selected = {table_name:table_name,title:title,field:field,row_index:selected["row_index"], cell:selected["cell"]};
		current_table = sanitizeName(table_name);
		current_col = {"title":title, "field":field};
		current_row = selected["row_index"];
	}
	refreshTables();
}

function deleteTable(table_name) {
	for (let i=0; i<data.length; i++) {
		if (data[i]["name"] == table_name) {
			data.splice(i, 1);
		}
	}
	refreshTables();
}

function deleteRow(table_name, index) {
	for (let table of data) {
		if (table["name"] == table_name) {
			table["rows"].splice(index, 1);
		}
	}
	refreshTables();
}

function deleteCol(table_name, index) {
	for (let table of data) {
		if (table["name"] == table_name) {
			table["col-def"].splice(index, 1);
		}
	}
	refreshTables();
}

function moveRow(table_name, old_index, new_index) {
	for (let table of data) {
		if (sanitizeName(table["name"]) == table_name) {
			let row = table["rows"][old_index];

			table["rows"].splice(old_index, 1);
			table["rows"].splice(new_index, 0, row);
		}
	}
	selectCell(selected["table_name"], selected["title"], selected["field"], selected["cell"]);
	refreshTables();
}

function moveCol(table_name, old_index, new_index) {
	for (let table of data) {
		if (sanitizeName(table["name"]) == table_name) {
			let col = table["col-def"][old_index];
			table["col-def"].splice(old_index, 1);
			table["col-def"].splice(new_index, 0, col);
		}
	}
	refreshTables();
}

function moveTable(old_index, new_index) {
	let table = data[old_index];
	data.splice(old_index, 1);
	data.splice(new_index, 0, table);
	refreshTables();
}

function drag(event, old_index) {
	event.originalEvent.dataTransfer.setData("text", old_index);
}

function rowDrag(event, old_index) {
	event.originalEvent.dataTransfer.setData("text", old_index);
}

function tableDrag(event, table_index) {
	event.originalEvent.dataTransfer.setData("text", table_index.toString());
}

function allowDrop(event) {
	event.preventDefault();
}

function colDrop(event, table_name, new_index) {
	event.preventDefault();
	let old_index = event.originalEvent.dataTransfer.getData("text");
	moveCol(table_name, parseInt(old_index), new_index);
}

function rowDrop(event, table_name, new_index) {
	event.preventDefault();
	let old_index = event.originalEvent.dataTransfer.getData("text");
	moveRow(table_name, parseInt(old_index), new_index);
}

function tableDrop(event, new_index) {
	event.preventDefault();
	let old_index = event.originalEvent.dataTransfer.getData("text");
	moveTable(parseInt(old_index), new_index);
}

function showTable(table_name) {
	let reset = false;
	if (current_table != table_name) {
		current_table = table_name;
		reset = true;
	}

	let tables = $(".editor-table");
	tables.css("display", "none");
	$(document.getElementById(`${table_name}-table`))
		.css("display", "block");
	let table_description = $(".table-description");
	table_description.css("display", "none");
	$(document.getElementById(`${table_name}-description`))
		.css("display", "block");
	let tabs = $(".tablink");
	tabs.removeClass("w3-grey");
	$(document.getElementById(`${table_name}-nav`))
		.addClass("w3-grey");
	for (let table of data) {
		if (sanitizeName(table["name"]) == table_name) {
			if (reset) {
				let child = $(document.getElementById(table_name+"-table"))
					.children(":nth-child(2)").children(":nth-child(2)");
				selectCell(table["name"], table["col-def"][0]["title"], table["col-def"][0]["field"], child.attr("id"));
			} else {
				let elm = $(document.getElementById(selected["cell"]));
				selectCell(table["name"], current_col["title"], current_col["field"], selected["cell"]);
			}
		}
	}
}

function startEditCell(cell_id) {
	let cell = $(document.getElementById(cell_id));
	let cell_text = $("#cell-text").val();
	let td = $("<td></td>")
		.attr("id", cell_id)
		.addClass("w3-green text-edit");
	let text_area = $(`<textarea></textarea>`)
		.on("focusout", (event) => {
			finishEditCell(event, cell_id);
		})
		.attr("id", cell_id+"-edit")
		.on("keydown", (event) => {
			finishEditCell(event, cell_id);
		})
		.text(cell_text);
	td.append(text_area);
	cell.parent().css("height", `${cell.outerHeight()}px`);
	cell.replaceWith(td);
	text_area.focus();
}

function startEditHeader(cell_id) {
	let cell = $(document.getElementById(cell_id));
	let cell_text = $("#col-name").val();
	let th = $(`<th></th>`)
		.attr("id", cell_id)
		.addClass("w3-green text-edit");
	let text_area = $("<textarea></textarea>")
		.on("focusout", (event) => {
			finishEditHeader(event, cell_id);
		})
		.attr("id", cell_id+"-edit")
		.on("keydown", (event) => {
			finishEditHeader(event, cell_id);
		})
		.text(cell_text);
	th.append(text_area);
	cell.parent().css("height", `${cell.outerHeight()}px`);
	cell.replaceWith(th);
	text_area.focus();
}

function finishEditCell(event, cell_id) {
	let cell = $(document.getElementById(`${cell_id}-edit`));
	if (event.ctrlKey && event.key == "Enter") {
		cell.off("focusout");
		let text_area_val = cell.val();
		$("#cell-text").val(text_area_val);
		save();
	} else if (event.key == "Escape") {
		cell.off("focusout");
		let cell_val = $("#cell-text").val();
		cell.val(cell_val);
		save();
	} else if (event.type == "focusout") {
		let text_area_val = cell.val();
		$("#cell-text").val(text_area_val);
		save();
	}
}

function finishEditHeader(event, cell_id) {
	let cell = $(document.getElementById(`${cell_id}-edit`));
	if (event.ctrlKey && event.key == "Enter") {
		cell.off("focusout");
		let text_area_val = $(document.getElementById(`${cell_id}-edit`)).val();
		$("#col-name").val(text_area_val);
		save();
	} else if (event.key == "Escape") {
		cell.off("focusout");
		let cell_val = $("#col-name").val();
		$(document.getElementById(`${cell_id}-edit`)).val(cell_val);
		save();
	} else if (event.type == "focusout") {
		let text_area_val = $(document.getElementById(`${cell_id}-edit`)).val();
		$("#col-name").val(text_area_val);
		save();
	}
}

function finishEditDescription(event, table_name) {
	let cell = $(document.getElementById(`${table_name}-description`));
	if (event.ctrlKey && event.key == "Enter") {
		cell.off("focusout");
		// let text_area_val = $(`#${cell_id}-edit`).val();
		// $("#col-name").val(text_area_val);
		save();
	} else if (event.key == "Escape") {
		cell.off("focusout");
		// let cell_val = $("#col-name").val();
		// $(`#${cell_id}-edit`).val(cell_val);
		save();
	} else if (event.type == "focusout") {
		// let text_area_val = $(`#${cell_id}-edit`).val();
		// $("#col-name").val(text_area_val);
		save();
	}
}

function navContext(event, table_name) {
	let context_menu = $(`<div></div>`)
		.css({"position": "absolute", "top": `${event.originalEvent.y-10}px`, "left": `${event.originalEvent.x-10}px`})
		.on("mouseleave", () => {
			context_menu.remove();
		});
	let buttons = $("<div></div>")
		.addClass("w3-dropdown-content w3-bar-block w3-border")
		.css("display", "block")
		.append($("<button>duplicate</button>")
			.addClass("w3-bar-item w3-button")
			.on("click", () => {
				duplicateTable(table_name);
			}));
	context_menu.append(buttons)
	$("body").append(context_menu);
}

function duplicateTable(table_name) {
	for (let table of data) {
		if (table["name"] == table_name) {
			addTable(table);
		}
	}
}

// function addSubTable(table_name) {
// 	for (let table of data) {
// 		if (table["name"] == table_name) {
// 			addTable(table);
// 		}
// 	}
// }

function saveFile(new_file) {
	ipcRenderer.send("save", data, new_file);
}

function openFile(file_path) {
	$.ajax({
		dataType: "json",
		url: file_path,
		cache: false,
		success: function(json, textStatus, jqXHR ) {
			data = json;
			refreshTables();
			showTable(sanitizeName(data[0]["name"]));
		}
	});
}

async function updateRecentFiles(new_file) {
	let recent_files = await ipcRenderer.invoke("getStoreValue", "recent_files");
	if (recent_files.includes(new_file)) {
		recent_files.splice(recent_files.indexOf(new_file), 1);
	}
	recent_files.splice(0, 0, new_file);
	ipcRenderer.invoke("setStoreValue", "recent_files", recent_files);
}

ipcRenderer.on("save", (event, new_file) => {
	saveFile(new_file);
})

ipcRenderer.on("open", (event, file_path) => {
	updateRecentFiles(file_path);
	openFile(file_path);
})